angular.module('starter.controllers')

.controller('FinishedCtrl', function($scope, $ionicLoading, $timeout, IDBuilder, SavedData,
  $cordovaSocialSharing, $cordovaFile, dataURItoBlob, $cordovaToast, storePageURL, AdHelper,
  $cordovaFileOpener2, $ionicPopup, galleryPath) {

  $scope.android = ionic.Platform.isAndroid();
  $scope.viewTitle = ionic.Platform.isAndroid() ? 'Preview' :
                     ionic.Platform.isIOS() ?     '' :
                                                  'View';
  if(typeof analytics !== 'undefined') { analytics.trackView("Fake ID Results"); }

  var filename;
  $scope.saved = false;
  $scope.savedData = SavedData.get();

  $scope.$on('$ionicView.beforeEnter', function(viewInfo, state) {
    $scope.saved = false;
    if ($scope.savedData.selection) {
      showLoading();
      filename = updateFileName();
      IDBuilder.build($scope.savedData.selection + '.json').then(onImageBuilt);
    } else {
      $cordovaToast.showLongCenter('Please go back and select a template.');
    }
  });

  $scope.$on('$ionicView.leave', function(e) {
    AdHelper.showInterstitial();
  });

  $scope.openHelp = function() {
    var alertPopup = $ionicPopup.alert({
     scope: $scope,
     title: 'Help',
     templateUrl: 'templates/finished-help.html'
   });
  };

  $scope.open = function() {
    var path = (ionic.Platform.isAndroid()) ? cordova.file.externalRootDirectory :
                (ionic.Platform.isIOS()) ? cordova.file.documentsDirectory :
                null;
    path += 'FakeIdImages/' + filename.split('-')[1];
    console.log('trying to open ' + path);
    $cordovaFileOpener2.open(path, "image/jpeg").then(function() {
      console.log('success');
    }, function(reason) {
      console.log('fail', reason);
    });
  };

  $scope.share = function() {
    showLoading();

    if(typeof analytics !== 'undefined') { analytics.trackEvent("Button", "pressed", "Share"); }

    if (ionic.Platform.isIOS()) {
      $cordovaFile.writeFile(cordova.file.cacheDirectory, "temp.jpg", dataURItoBlob($scope.finished), true)
      .then(function(success) {
        $cordovaSocialSharing.share(
          null,
          filename,
          cordova.file.cacheDirectory + "temp.jpg",
          null
        ).then(hideLoading, hideLoading);
      });

    } else if (ionic.Platform.isAndroid()) {
      //android cache dir isn't always created so it's better to use the tried and true data url instead
      $cordovaSocialSharing.share(
        null,
        filename,
        $scope.finished,
        null
      ).then(hideLoading, hideLoading);
    }


  };

  $scope.save = function() {
    $ionicLoading.show({
      templateUrl: "templates/loading.html",
      duration: 10000
    });

    if(typeof analytics !== 'undefined') { analytics.trackEvent("Button", "pressed", "Save"); }

    var root = (ionic.Platform.isAndroid()) ? cordova.file.externalRootDirectory :
                (ionic.Platform.isIOS()) ? cordova.file.documentsDirectory :
                null;
    if (root === null) return;
    var filepath = galleryPath;

    //create album directory if it doesn't exist, then write the image into it
    $cordovaFile.createDir(root, "FakeIdImages", false)
    .then(moveFileToAlbumDir, moveFileToAlbumDir)
    .then(saveSuccess, saveFailed);

    function moveFileToAlbumDir() {
      return $cordovaFile.writeFile(filepath, filename.split('-')[1], dataURItoBlob($scope.finished), false);
    }
    function saveSuccess() {
      $scope.saved = true;
      $ionicLoading.hide();
      $cordovaToast.showShortCenter('Saved to Gallery!');
    }
    function saveFailed(error) {
      $ionicLoading.hide();
      switch (error.code) {
        case 12:
          $cordovaToast.showShortCenter('Already Saved.');
          break;
        default:
          $cordovaToast.showShortCenter('Save Failed (Error Code ' + error.code + ')');
          break;
      }

    }
  };

  function onImageBuilt(src) {
    $scope.finished = src;
    hideLoading();
  }

  function showLoading() {
      $ionicLoading.show({
        templateUrl: "templates/loading.html",
        duration: 10000
      });
  }

  function hideLoading() {
    $ionicLoading.hide();
  }

  function updateFileName() {
    var date = new Date();
    var name = "Image-" +
      date.getFullYear() +
      date.getMonth() +
      date.getDate() +
      date.getHours() +
      date.getMinutes() +
      date.getSeconds() +
      ".jpg";
      return name;
  }

});
