angular.module('starter.controllers')
  .controller('PhotoCtrl', function($scope, SavedData, $location, $ionicLoading, $ionicPopup, $cordovaCamera, $ionicModal, $localStorage, $timeout) {

    var canvas, image, baseScale;
    var defaultFile = "img/blue_outline.png";
    var rotation = 0,
      flip = 1;

    $scope.canvasStyle = {
      width: "177px",
      height: "205px"
    };

    $scope.savedData = SavedData.get();

    $ionicModal.fromTemplateUrl('templates/photo-modal.html', {
      scope: $scope,
      animation: 'slide-in-up'
    }).then(function(modal) {
      $scope.modal = modal;
      if (ionic.Platform.isAndroid() && $localStorage.restorePhoto) {
        $timeout(function() {
          $scope.modal.show();
        });
      }
    });

    $scope.showModal = function() {
      $scope.modal.show();
    };
    $scope.hideModal = function() {
      $scope.modal.hide();
    };

    $scope.$on('modal.shown', function(viewInfo, state) {
      var canvasEl = angular.element($scope.modal.el.querySelector('#photoCanvas'))[0];
      canvas = canvas || new fabric.StaticCanvas(canvasEl);
      canvas.setDimensions({width: '177px', height: '203px'}, {cssOnly: true});

      $scope.savedData.inputs.photo = $scope.savedData.inputs.photo || defaultFile;

      //if we're trying to restore from the main activity being killed while taking a photo
      if (ionic.Platform.isAndroid() && $localStorage.restorePhoto === true) {

        console.log('entered restore block');
        navigator.camera.checkForSavedResult(function(im) {
          console.log('saved result returned ' + im);
          if (im) {
            loadImage(im);
            restoreSuccess();
          } else {
            loadImage($scope.savedData.inputs.photo);
            restoreFail();
          }
        }, function(reason) {
          console.log('saved result failed because ' + reason);
          loadImage($scope.savedData.inputs.photo);
          //show error if the reason was not a cancel
          if (reason.indexOf('cancel') === -1) restoreFail();
        }, $localStorage.photoOptions);
      }

      else {
        loadImage($scope.savedData.inputs.photo);
      }

      $localStorage.restorePhoto = false;
    });

    $scope.$on('modal.hidden', function(viewInfo, state) {
      $scope.save();
      $localStorage.restorePhoto = false;
    });

    function loadImage(url) {
      fabric.Image.fromURL(url, function(oImg) {
        baseScale = calculateBaseScale(oImg);
        oImg.set({
          originX: 'center',
          originY: 'center',
          left: canvas.width / 2,
          top: canvas.height / 2,
          width: oImg.width * baseScale,
          height: oImg.height * baseScale
        });
        canvas.add(oImg);
        image = oImg;
      });
    }

    $scope.fromCamera = function() {
      var options = {
        destinationType: Camera.DestinationType.FILE_URI,
        sourceType: Camera.PictureSourceType.CAMERA,
        mediaType: Camera.MediaType.PICTURE,
        encodingType: Camera.EncodingType.JPEG,
        targetWidth: 354,
        targetHeight: 405,
        //allowEdit: true,
        saveToPhotoAlbum: false
      };

      $localStorage.restorePhoto = true;
      $localStorage.photoOptions = options;
      $timeout(function() {
        $cordovaCamera.getPicture(options).then(imageSuccess, imageFail);
      });

    };

    $scope.fromGallery = function() {
      var options = {
        destinationType: Camera.DestinationType.FILE_URI,
        sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
        mediaType: Camera.MediaType.PICTURE,
        encodingType: Camera.EncodingType.JPEG,
        targetWidth: 354,
        targetHeight: 405
        // allowEdit: true
      };

      $localStorage.restorePhoto = true;
      $localStorage.photoOptions = options;
      $timeout(function() {
        $cordovaCamera.getPicture(options).then(imageSuccess, imageFail);
      });

    };

    function imageSuccess(fileURI) {
      $ionicLoading.show({
        templateUrl: "templates/loading.html"
      });
      var file = fileURI + "?date=" + encodeURIComponent(Date.now());
      console.log("image success: " + file);
      image.setSrc(file, function() {
        recalculate();
        canvas.renderAll();
        $ionicLoading.hide();
      });
      $localStorage.restorePhoto = false;
    }

    function imageFail(message) {
      console.log('Retrieving image failed: ' + message);
    }

    $scope.rotate = function() {
      image.angle += 90;
      var scale = 0;
      var tall = (image.getHeight() > image.getWidth());
      var upright = (image.angle % 180 === 0);
      if (tall) {
        if (upright) {
          scale = canvas.width / image.width;
        } else {
          scale = canvas.height / image.width;
        }
      } else {
        if (upright) {
          scale = canvas.height / image.height;
        } else {
          scale = canvas.width / image.height;
        }
      }
      image.scaleX = scale;
      image.scaleY = scale;
      canvas.renderAll();
    };

    $scope.flip = function() {
      var upright = (image.angle % 180 === 0);
      if (upright) {
        image.flipX = !image.flipX;
      } else {
        image.flipY = !image.flipY;
      }
      canvas.renderAll();
    };

    $scope.reset = function() {
      image.setSrc(defaultFile, function() {
        recalculate();
        canvas.renderAll();
      });
    };

    $scope.save = function() {
      $scope.savedData.inputs.photo = canvas.toDataURL({
        format: 'jpeg',
        quality: '1'
      });
      canvas.remove(image);
      $cordovaCamera.cleanup()
      .then(function(success) {
        console.log("Camera Cleanup succeeded.");
      }, function(fail) {
        console.log("Camera Cleanup failed because " + fail);
      });
    };

    function recalculate() {
      baseScale = calculateBaseScale(image);
      image.flipX = false;
      image.flipY = false;
      image.angle = 0;
      image.left = canvas.width / 2;
      image.top = canvas.height / 2;
      image.scaleX = baseScale;
      image.scaleY = baseScale;
    }

    function calculateBaseScale(img) {
      var hRatio = canvas.width / img.width;
      var wRatio = canvas.height / img.height;
      var ratio = Math.max(hRatio, wRatio);
      return ratio;
    }

    //----------------------------------------------------------
    // We attempt a restore when the activity was killed when
    // retrieving a photo. It looks in the cache for an intermediate
    // save of the selected photo, and succeeds if it finds one.
    //----------------------------------------------------------
    function restoreSuccess() {
      if (typeof analytics !== 'undefined') {
        analytics.trackEvent("Photo Screen Restore", "success");
      }
    }

    function restoreFail() {
      if (typeof analytics !== 'undefined') {
        analytics.trackEvent("Photo Screen Restore", "fail");
      }
      $ionicPopup.alert({
        title: 'Woops...',
        template: 'The photo could not be delivered.  The error has been reported to the developer.',
        okType: 'button-assertive'
      });
    }

  });
