angular.module('starter.controllers')
  .controller('SignatureCtrl', function($scope, SavedData, $ionicModal, $timeout, $ionicLoading) {

    var container, canvas, strokeWidthDefault = 3;
    var savedData = SavedData.get();

    $ionicModal.fromTemplateUrl('templates/signature-modal.html', {
      scope: $scope,
      animation: 'slide-in-up'
    }).then(function(modal) {
      $scope.modal = modal;
    });

    $scope.showModal = function() {
      $scope.modal.show();
    };
    $scope.hideModal = function() {
      $scope.modal.hide();
    };

    $scope.$on('modal.shown', function(viewInfo, state) {
      var canvasEl = angular.element($scope.modal.el.querySelector('#canvas'))[0];
      canvas = canvas || new fabric.Canvas(canvasEl, {
        isDrawingMode: true
      });
      canvas.freeDrawingBrush.width = strokeWidthDefault;

      var containerEl = $scope.modal.el;
      container = angular.element(containerEl.querySelector('#signatureContainer'))[0];

      window.addEventListener('resize', waitToResize, false);

      resize();

      fabric.loadSVGFromString(savedData.inputs.signature, function(objects, options) {
        var group = fabric.util.groupSVGElements(objects, options);
        group.set({
          top: group.top * (canvas.height / group.height),
          left: group.left * (canvas.width / group.width),
          scaleY: canvas.height / group.height,
          scaleX: canvas.width / group.width
        });
        canvas.add(group);
        canvas.renderAll();
      });
    });

    $scope.$on('modal.hidden', function(viewInfo, state) {
      $scope.save();
      window.removeEventListener('resize', waitToResize, false);
    });

    function resize() {
      console.log('resizing signature canvas');

      var destWidth = 561;
      var destHeight = 89;
      var ratio = destHeight / destWidth;

      var oldW = canvas.getWidth();
      var oldH = canvas.getHeight();

      //container can't shrink if the canvas is wider than the window
      //so we have to shrink the canvas it and let container resize properly
      canvas.setDimensions({
        width: 1,
        height: 1
      });

      canvas.setDimensions({
        width: container.clientWidth,
        height: container.clientWidth * ratio
      });

      var factor = canvas.getWidth() / oldW;
      zoomObjects(factor);

      container.style.height = canvas.getHeight() + "px";
      canvas.freeDrawingBrush.width = strokeWidthDefault * (canvas.height / destHeight);
    }

    function waitToResize() {
      $ionicLoading.show({
        templateUrl: "templates/loading.html"
      });
      $timeout(function() {
        resize();
        $ionicLoading.hide();
      }, 1000);
    }

    function zoomObjects(factor) {
      var objects = canvas.getObjects();
      for (var i in objects) {
        var scaleX = objects[i].scaleX;
        var scaleY = objects[i].scaleY;
        var left = objects[i].left;
        var top = objects[i].top;

        var tempScaleX = scaleX * factor;
        var tempScaleY = scaleY * factor;
        var tempLeft = left * factor;
        var tempTop = top * factor;

        objects[i].scaleX = tempScaleX;
        objects[i].scaleY = tempScaleY;
        objects[i].left = tempLeft;
        objects[i].top = tempTop;

        objects[i].setCoords();
      }
      canvas.renderAll();
      canvas.calcOffset();
    }

    $scope.reset = function() {
      canvas.clear().renderAll();
    };

    $scope.save = function() {
      savedData.inputs.signature = canvas.toSVG();
      savedData.inputs.signaturePNG = canvas.toDataURL({
        format: 'png', multiplier: 2
      });
      canvas.clear().renderAll(); //clear it so next time we open the modal it redraws to scale
    };
  });
