angular.module('starter.controllers')
  .controller('TemplateCtrl', function($scope, SavedData, Templates, focus, $ionicScrollDelegate, $ionicPopup,
    storePageURL, $window, $cordovaSocialSharing, Prefs, AdHelper, $timeout) {

    if(typeof analytics !== 'undefined') { analytics.trackView("Fake ID Templates"); }

    $scope.savedData = SavedData.get();
    $scope.prefs = Prefs.get();
    $scope.searchText = "";
    $scope.searchMode = false;
    Templates.get().then(setTemplates);

    $scope.previewFile = ($scope.savedData.selection) ? $scope.savedData.selection + "_template.jpg" : "nothing.png";

    $scope.$on('$ionicView.beforeLeave', function(e) {
      $scope.saveSelection();
    });
    $scope.$on('$ionicView.leave', function(e) {
      if (typeof analytics !== 'undefined') {
        if ($scope.savedData.selection) {
          analytics.trackEvent("Template", "chosen", $scope.savedData.selection);
        }
      }
      AdHelper.showInterstitial();
    });

    $scope.setPreview = function(previewFile) {
      $scope.previewFile = previewFile;
    };

    $scope.saveSelection = function() {
      SavedData.save($scope.savedData);
    };

    $scope.$watch('savedData.selection', function(newVal, oldVal) {
      if (newVal && $scope.searchMode) {
        $scope.toggleSearch();
      }
    });

    $scope.toggleSearch = function() {
      $timeout(function() {
        if ($scope.prefs.listMode) { $scope.toggleListMode(); }
        $scope.searchMode = !$scope.searchMode;
        $scope.searchText = "";

        if ($scope.searchMode) {
          $ionicScrollDelegate.scrollTop();
          focus('search', true);
        } else {
          focus('search', false);
        }
      });
    };

    $scope.toggleListMode = function() {
      $timeout(function() {
        $scope.prefs.listMode = !$scope.prefs.listMode;
        Prefs.save($scope.prefs);
      }, 300);
    };

    $scope.showBonusDialog = function() {
      if(typeof analytics !== 'undefined') { analytics.trackEvent("Bonus Dialog", "shown"); }
      $ionicPopup.confirm({
        title: 'Unlock bonus templates',
        template: 'Rate or share the app to unlock 3 extra templates! They will be added to the bottom of the list.',
        buttons: [{
          text: 'Cancel',
          type: 'button-clear'
        },
        {
          text: 'Rate',
          type: 'button-positive',
          onTap: function() {
            if(typeof analytics !== 'undefined') { analytics.trackEvent("Button", "pressed", "Unlock Bonus Rate"); }
            storePageURL.open();
            successWithAlert();
          }
        }, {
          text: 'Share',
          type: 'button-positive',
          onTap: function() {
            if(typeof analytics !== 'undefined') { analytics.trackEvent("Button", "pressed", "Unlock Bonus Share"); }
            $cordovaSocialSharing.share("Make a fake ID with this free app!", "Fake ID Generator", null, storePageURL.www)
              .then(successWithAlert);
          }
        }]
      });

      function successWithAlert() {
        $scope.savedData.bonusUnlocked = true;
        if(typeof analytics !== 'undefined') { analytics.trackEvent("Template", "unlocked"); }
        $ionicPopup.alert({
          title: "Templates unlocked!",
          template: "3 new templates have been added to the bottom of the list. (FBI, Blueprint, Credit Card)",
          okType: "button-positive"
        });
      }
    };

    function setTemplates(templates) {
      $scope.templates = templates;
    }
  });
