angular.module('starter.services', [])

.factory('Templates', function($http, $q) {
  return {
    get: function() {
      var promise = $http.get('json/templates.json')
        .then(function(response) {
          console.log('successfully got Templates\'s Json');
          return response.data;
        }, function(reason) {
          console.error('http get failed in Templates service');
          $q.reject(reason);
        });
      return promise;
    }
  };
})

.factory('Params', function($http, $q) {
  return {
    get: function() {
      var promise = $http.get('json/params.json')
        .then(function(response) {
          console.log('successfully got Param\'s Json');
          return response.data;
        }, function(reason) {
          console.error('http get failed in Params service');
          $q.reject(reason);
        });
      return promise;
    }
  };
})

.factory('Prefs', function($localStorage) {
  if (!$localStorage.prefs) {
    $localStorage.prefs = {listMode: true};
  }
  return {
    get: function() {
      return $localStorage.prefs;
    },
    save: function(prefs) {
      $localStorage.prefs = prefs;
    }
  };
})

.factory('SavedData', function($localStorage) {
  return {
    get: function() {
      if (!$localStorage.savedData) {
        $localStorage.savedData = {
          inputs: {
            texts: {}
          }
        };
      }
      return $localStorage.savedData;
    },
    getInputs: function() {
      return $localStorage.savedData.inputs || {
        texts: {}
      };
    },
    reset: function() {
      $localStorage.$reset({
        inputs: {
          texts: {}
        }
      });
    },
    clearInputs: function() {
      $localStorage.savedData.inputs = {
        texts: {}
      };
    },
    clearText: function(paramName) {
      $localStorage.savedData.inputs.texts[paramName] = "";
    },
    save: function(savedData) {
      $localStorage.savedData = savedData;
    }
  };
})

.factory('IDs', function($http, $q) {

  return {
    get: function(jsonFileName) {
      var promise = $http.get('json/ids/' + jsonFileName)
        .then(function(response) {
          console.log('successfully got ID\'s Json');
          return response.data;
        }, function(reason) {
          console.error('http get failed in IDs service');
          $q.reject(reason);
        });
      return promise;
    }
  };
})

.factory('focus', function($timeout, $window) {
  return function(id, focus) {
    // timeout makes sure that is invoked after any other event has been triggered.
    // e.g. click events that need to run before the focus or
    // inputs elements that are in a disabled state but are enabled when those events
    // are triggered.
    $timeout(function() {
      var element = $window.document.getElementById(id);
      if (element)
        if (focus) {
          element.focus();
        } else {
          element.blur();
        }
    });
  };
})

.factory('dataURItoBlob', function() {
  return function(dataURI) {
    var byteString;
    var byteStringParts = dataURI.split(',');
    if (byteStringParts[0].indexOf('base64') >= 0) {
      //if dataURI is in base64 format
      byteString = atob(byteStringParts[1]);
    } else {
      byteString = decodeURIComponent(byteStringParts[1]);
    }

    var mimeString = byteStringParts[0].split(':')[1].split(';')[0];
    //ex: 'image:jpeg'

    var intArray = new Uint8Array(byteString.length);
    for (var i = 0; i < byteString.length; i++) {
      intArray[i] = byteString.charCodeAt(i);
    }

    return new Blob([intArray], {type: mimeString});
  };
})

.factory('storePageURL', function($cordovaInAppBrowser) {
  var url = { www: "http://www.google.com", native: "http://www.bing.com" };
  document.addEventListener('deviceready', function() {
    if (ionic.Platform.isIOS()) {
      url.www = "https://itunes.apple.com/us/app/fake-id-generator/id1070766884?mt=8";
      url.native = "https://itunes.apple.com/us/app/fake-id-generator/id1070766884?mt=8";
    } else if (ionic.Platform.isAndroid()) {
      if (ionic.Platform.platform().indexOf('amazon') > -1 ||
          device.manufacturer.indexOf('amazon') > -1) {
        url.www = "http://www.amazon.com/gp/mas/dl/android?p=app.bugbyte.fakeidgenerator";
        url.native = "amzn://apps/android?p=app.bugbyte.fakeidgenerator";
      } else {
        url.www = "https://play.google.com/store/apps/details?id=app.bugbyte.fakeidgenerator";
        url.native = "market://details?id=app.bugbyte.fakeidgenerator";
      }
    }
  });

  url.open = function() {
    $cordovaInAppBrowser.open(url.native, "_system");
  };
  return url;
})

.factory('galleryPath', function() {
  var root, path, dir;
  dir = "FakeIdImages";
  if (ionic.Platform.isAndroid()) {
    root = cordova.file.externalRootDirectory;
    // dir = "/FakeIdImages";
  } else if (ionic.Platform.isIOS()) {
    root = cordova.file.documentsDirectory;
    // dir = "FakeIdImages";
  }
  var path = root + dir;
  console.log("galleryPath returning: " + path);
  return path;
})
;
