angular.module('starter.services')

.factory('ImageEffects', function($q) {

  return {
    grayscale: function(render) {
      var defer = $q.defer();

      var canvas = document.createElement('canvas');
      var context = canvas.getContext('2d');

      canvas.width = render.w;
      canvas.height = render.h;

      context.drawImage(render.img, 0, 0, canvas.width, canvas.height);

      var imgData = context.getImageData(0, 0, canvas.width, canvas.height);
      var pixels  = imgData.data;
      for (var i = 0, n = pixels.length; i < n; i += 4) {
        var grayscale = pixels[i] * 0.3 + pixels[i+1] * 0.59 + pixels[i+2] * 0.11;
        pixels[i  ] = grayscale;        // red
        pixels[i+1] = grayscale;        // green
        pixels[i+2] = grayscale;        // blue
        //pixels[i+3]                   is alpha
      }
      context.putImageData(imgData, 0, 0, 0, 0, canvas.width, canvas.height);

      render.img.onload = function() {
        defer.resolve();
      };
      render.img.src = canvas.toDataURL();

      return defer.promise;
    },

    invertColor: function(render) {
      var defer = $q.defer();

      var canvas = document.createElement('canvas');
      var context = canvas.getContext('2d');

      canvas.width = render.w;
      canvas.height = render.h;

      context.drawImage(render.img, 0, 0, canvas.width, canvas.height);

      var imgData = context.getImageData(0, 0, canvas.width, canvas.height);
      var pixels  = imgData.data;
      for (var i = 0, n = pixels.length; i < n; i += 4) {
        pixels[i  ] = (255-pixels[i]);
        pixels[i+1] = (255-pixels[i+1]);
        pixels[i+2] = (255-pixels[i+2]);

      }
      context.putImageData(imgData, 0, 0, 0, 0, canvas.width, canvas.height);

      render.img.onload = function() {
        defer.resolve();
      };
      render.img.src = canvas.toDataURL();

      return defer.promise;
    },

    colorFilter: function(render, color) {
      var defer = $q.defer();

      var canvas = document.createElement('canvas');
      var context = canvas.getContext('2d');

      canvas.width = render.w;
      canvas.height = render.h;

      R = HexToR(color);
      G = HexToG(color);
      B = HexToB(color);

      function HexToR(h) {return parseInt((cutHex(h)).substring(0,2),16);}
      function HexToG(h) {return parseInt((cutHex(h)).substring(2,4),16);}
      function HexToB(h) {return parseInt((cutHex(h)).substring(4,6),16);}
      function cutHex(h) {return (h.charAt(0)=="#") ? h.substring(1,7):h;}

      context.drawImage(render.img, 0, 0, canvas.width, canvas.height);

      var imgData = context.getImageData(0, 0, canvas.width, canvas.height);
      var pixels  = imgData.data;
      for (var i = 0, n = pixels.length; i < n; i += 4) {
        pixels[i  ] = pixels[i  ] * (R/255);
        pixels[i+1] = pixels[i+1] * (G/255);
        pixels[i+2] = pixels[i+2] * (B/255);

      }
      context.putImageData(imgData, 0, 0, 0, 0, canvas.width, canvas.height);

      render.img.onload = function() {
        defer.resolve();
      };
      render.img.src = canvas.toDataURL();

      return defer.promise;
    }
  };

});
